/* gnubiff -- a mail notification program
 * Copyright (C) 2000-2003 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <sstream>
#include <cstdio>
#include <glib.h>
#include "Setup.h"
#include "Biff.h"
#include "Mailbox.h"
#include "File.h"
#include "Mh.h"
#include "Maildir.h"
#include "Imap4.h"
#include "Pop3.h"
#include "Apop.h"
#include "Popup.h"



// ===================================================================
// = Static attributes ===============================================
// ===================================================================


// ===================================================================
// = Callbacks "C" binding ===========================================
// ===================================================================
extern "C" {
  //! C binding
  void SETUP_on_cancel (GtkWidget *widget, gpointer data)		{((Setup *) data)->on_cancel ();}
  //! C binding
  void SETUP_on_save (GtkWidget *widget, gpointer data)			{((Setup *) data)->on_save ();}
  //! C binding
  void SETUP_on_quit (GtkWidget *widget, gpointer data)			{((Setup *) data)->on_quit ();}
  //! C binding
  void SETUP_on_suspend (GtkWidget *widget, gpointer data)		{((Setup *) data)->on_suspend ();}
  //! C binding
  void SETUP_on_tree_select (GtkTreeSelection *sel, gpointer data)	{((Setup *) data)->on_tree_select (sel);}
  //! C binding
  void SETUP_on_protocol (GtkWidget *widget, gpointer data)		{((Setup *) data)->on_protocol ();}
  //! C binding
  void SETUP_on_sound (GtkWidget *widget, gpointer data)		{((Setup *) data)->on_sound ();}
  //! C binding
  void SETUP_on_misc_change (GtkWidget *widget, gpointer data)		{((Setup *) data)->on_misc_change ();}
  //! C binding
  void SETUP_on_browse_address (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_address ();}
  //! C binding
  void SETUP_on_browse_certificate (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_certificate ();}
  //! C binding
  void SETUP_on_browse_sound (GtkWidget *widget, gpointer data)		{((Setup *) data)->on_browse_sound ();}
  //! C binding
  void SETUP_on_play_sound (GtkWidget *widget, gpointer data)		{((Setup *) data)->on_play_sound ();}
  //! C binding
  void SETUP_on_browse_mail_app (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_mail_app ();}
  //! C binding
  void SETUP_on_browse_mail_image (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_mail_image ();}
  //! C binding
  void SETUP_on_browse_nomail_image (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_nomail_image ();}

  //! C binding
  void SETUP_on_browse_popup_title_font (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_font ("popup_title_font");}
  //! C binding
  void SETUP_on_browse_popup_body_font (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_font ("popup_body_font");}
  //! C binding
  void SETUP_on_browse_biff_title_font (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_font ("biff_title_font");}
  //! C binding
  void SETUP_on_browse_biff_body_font (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_font ("biff_body_font");}
  //! C binding
  void SETUP_on_browse_applet_title_font (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_font ("applet_title_font");}
  //! C binding
  void SETUP_on_browse_applet_body_font (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_font ("applet_body_font");}

  //! C binding
  void SETUP_on_browse_popup_title_color (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_color ("popup_title_color");}
  //! C binding
  void SETUP_on_browse_popup_body_color (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_color ("popup_body_color");}
  //! C binding
  void SETUP_on_browse_biff_title_color (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_color ("biff_title_color");}
  //! C binding
  void SETUP_on_browse_biff_body_color (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_color ("biff_body_color");}
  //! C binding
  void SETUP_on_browse_applet_title_color (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_color ("applet_title_color");}
  //! C binding
  void SETUP_on_browse_applet_body_color (GtkWidget *widget, gpointer data)	{((Setup *) data)->on_browse_color ("applet_body_color");}
}


// ===================================================================
// = Setup ===========================================================
// ===================================================================
Setup::Setup (Biff *owner, string xmlFilename) : GUI (xmlFilename) {
  _owner = owner;
}


// ===================================================================
// = ~Setup ==========================================================
// ===================================================================
Setup::~Setup (void) {
}

// ===================================================================
// = create ==========================================================
/**
  Create the interface (GUI::create()) and build the tree menu
  @return true
*/
// ===================================================================
gint Setup::create (void) {
  GUI::create();

  // Hide tabs from the notebook
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK(get("Notebook")), FALSE);

  // Make event box under label to be prelighted (cosmetic only)
  gtk_widget_set_state (get("NotebookEventBox"), GTK_STATE_PRELIGHT);

  // Create the tree on the left
  GtkTreeStore *	tree;
  GtkWidget *		treeview;
  GtkTreeViewColumn *	column;
  GtkCellRenderer   *	cell;
  GtkTreeSelection  *	sel;
  GtkTreeIter		top_iter, child_iter;

  // Create a new tree store with 5 cells:
  tree = gtk_tree_store_new (5,
			     GDK_TYPE_PIXBUF,	// Small image displayed directly in the tree entry
			     G_TYPE_STRING,	// Label displayed directly in the tree entry
			     G_TYPE_INT,	// Page index related to this tree entry
			     G_TYPE_STRING,	// Notebook label that will be displayed
			     GDK_TYPE_PIXBUF);	// Big image to be displayed next to notebook label
  
  // Create a tree view for the tree...
  treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree));
  //  ... and we do not want to see headers in
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);

  // Create a column template...
  column = gtk_tree_view_column_new ();
  //  ... first cell is the pixbuf...
  cell = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_column_pack_start (column, cell, FALSE);
  gtk_tree_view_column_set_attributes (column, cell, "pixbuf", 0, NULL);
  // ... second cell is the label
  cell = gtk_cell_renderer_text_new ();
  gtk_tree_view_column_pack_start (column, cell, TRUE);
  gtk_tree_view_column_set_attributes (column, cell, "text", 1, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

  // We put our treeview within the tree frame
  gtk_container_add (GTK_CONTAINER (get("TreeFrame")), treeview);

  // We connect our selection callback
  sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
  g_signal_connect (G_OBJECT (sel), "changed",  G_CALLBACK (SETUP_on_tree_select), this);

  // Now we add branches to the tree
  addTreeBranch (tree, "Mailbox", GNUBIFF_DATADIR"/BiffMailbox.png", "Mailbox configuration", NULL, &top_iter, 0);
  gtk_tree_selection_select_iter (sel, &top_iter);
  addTreeBranch (tree, "Options", GNUBIFF_DATADIR"/BiffOptions.png", "Options", &top_iter, &child_iter, 1);
  addTreeBranch (tree, "Layout", GNUBIFF_DATADIR"/BiffLayout.png", "Layout", NULL, &top_iter, 2);
  addTreeBranch (tree, "Geometry", GNUBIFF_DATADIR"/BiffGeometry.png", "Geometry", &top_iter, &child_iter, 3);
  addTreeBranch (tree, "Font", GNUBIFF_DATADIR"/BiffFont.png", "Font", &top_iter, &child_iter, 4);
  addTreeBranch (tree, "Credits", GNUBIFF_DATADIR"/BiffCredits.png", "Credits", NULL, &top_iter, 5);

  // We expand the tree so all branches are visible...
  gtk_tree_view_expand_all (GTK_TREE_VIEW (treeview));

  // ... and we show it
  gtk_widget_show_all (treeview);

  // We don't need this object anymore
  g_object_unref (G_OBJECT (tree));

  // Make thanks text smaller
  PangoFontDescription *font;
  font = pango_font_description_from_string ("Sans 8");
  gtk_widget_modify_font (get("ThanksText"), font);
  pango_font_description_free (font);

  // Here we attach some data to widgets that cannot contain
  // enough information by themselves. For example the sound button
  // only contains name of the basename of the file (without path), so
  // we need to store fullname somewhere else.
  g_object_set_data (G_OBJECT(get("soundfile")), "_file_", gpointer (0));
  g_object_set_data (G_OBJECT(get("mail_image")), "_file_", gpointer (0));
  g_object_set_data (G_OBJECT(get("nomail_image")), "_file_", gpointer (0));
  g_object_set_data (G_OBJECT(get("popup_title_color")), "_color_", gpointer (0));
  g_object_set_data (G_OBJECT(get("popup_body_color")), "_color_", gpointer (0));
  g_object_set_data (G_OBJECT(get("biff_title_color")), "_color_", gpointer (0));
  g_object_set_data (G_OBJECT(get("biff_body_color")), "_color_", gpointer (0));
  g_object_set_data (G_OBJECT(get("applet_title_color")), "_color_", gpointer (0));
  g_object_set_data (G_OBJECT(get("applet_body_color")), "_color_", gpointer (0));


  // Now we make some widgets sensitive or not depending on USE_GNOME
#ifdef USE_GNOME
  if (_owner->_mode == GNOME_MODE) {
    gtk_widget_set_sensitive (get("biff_geometry"), FALSE);
    gtk_widget_set_sensitive (get("biff_font"), FALSE);
    gtk_widget_set_sensitive (get("quit"), FALSE);
  }
  else
    gtk_widget_set_sensitive (get("applet_font"), FALSE);
#else
  gtk_widget_set_sensitive (get("applet_font"), FALSE);
#endif

  // If we don't have crypto, we can't use apop
#ifndef HAVE_LIBCRYPTO
  gtk_widget_set_sensitive (get("apop"), FALSE);
#endif

  // If we don't have ssl, we can't use it
#ifndef HAVE_LIBSSL
  gtk_widget_set_sensitive (get("ssl"), FALSE);
  gtk_widget_set_sensitive (get("CertificateTable"), FALSE);
#endif

  // That's it
  return true;
}


// ===================================================================
// = update ==========================================================
/**
  update Setup according to _owner
*/
// ===================================================================
void Setup::update (void) {
  GdkPixbufAnimation *animation;
  //
  // Nothing special, we read settings from _owner and set corresponding
  //  widgets accordingly.
  // Very boring code indeed !
  //
  // Mailbox
  gtk_entry_set_text (GTK_ENTRY (get("title")), _owner->_title.c_str());
  string protocol = "file";
  switch (_owner->_protocol) {
  case PROTOCOL_MH:
    protocol = "mh";
    break;
  case PROTOCOL_MAILDIR:
    protocol = "maildir";
    break;
  case PROTOCOL_POP3:
    protocol = "pop3";
    break;
  case PROTOCOL_APOP:
    protocol = "apop";
    break;
  case PROTOCOL_IMAP4:
    protocol = "imap4";
    break;
  }
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get(protocol.c_str())), TRUE);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("port")), _owner->_port);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("ssl")), _owner->_ssl);
  gtk_entry_set_text (GTK_ENTRY (get("address")), _owner->_address.c_str());
  gtk_entry_set_text (GTK_ENTRY (get("user")), _owner->_user.c_str());
  gtk_entry_set_text (GTK_ENTRY (get("password")), _owner->_password.c_str());
  gtk_entry_set_text (GTK_ENTRY (get("certificate")), _owner->_certificate.c_str());


  // Options
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("polltime")), _owner->_polltime);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("poptime")), _owner->_poptime);
  string sound_type = "nosound";
  switch (_owner->_sound_type) {
  case SOUND_BEEP:
    sound_type = "beep";
    break;
  case SOUND_FILE:
    sound_type = "sound";
    break;
  }
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get(sound_type.c_str())), TRUE);
  gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("soundfile")))),
		      g_path_get_basename (_owner->_sound_file.c_str()));
  // Here, we attach the full soundfile name to the widget, taking care of previous attached data
  gchar *data = (gchar *) g_object_get_data (G_OBJECT(get("soundfile")), "_file_");
  if (data)
    g_free (data);
  data = g_strdup (_owner->_sound_file.c_str());
  g_object_set_data (G_OBJECT(get("soundfile")), "_file_", gpointer (data));
  gtk_range_set_value (GTK_RANGE (get ("volume")), _owner->_sound_volume);  
  if (_owner->_state == STATE_RUNNING) {
    gtk_label_set_markup (GTK_LABEL (get("suspend_label")), "<span color=\"darkgreen\"><b>Running</b></span>");
    gtk_image_set_from_stock (GTK_IMAGE (get("suspend_image")), GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON);
  }
  else {
    gtk_label_set_markup (GTK_LABEL (get("suspend_label")), "<span color=\"red\"><b>Stopped</b></span>");
    gtk_image_set_from_stock (GTK_IMAGE (get("suspend_image")), GTK_STOCK_STOP, GTK_ICON_SIZE_BUTTON);
  }


  // Advanced
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("display_popup")), _owner->_display_popup); 
  gtk_entry_set_text (GTK_ENTRY (get("mail_app")), _owner->_mail_app.c_str());



  // Layout
  animation = gdk_pixbuf_animation_new_from_file (_owner->_mail_image.c_str(), 0);
  gtk_image_set_from_animation (GTK_IMAGE (gtk_bin_get_child (GTK_BIN(get("mail_image")))), animation);
  if (animation)
    g_object_unref (animation);
  // Here, we attach the full filename to the widget, taking care of previous attached data
  data = (gchar *) g_object_get_data (G_OBJECT(get("mail_image")), "_file_");
  if (data)
    g_free (data);
  data = g_strdup (_owner->_mail_image.c_str());
  g_object_set_data (G_OBJECT(get("mail_image")), "_file_", gpointer (data));


  animation = gdk_pixbuf_animation_new_from_file (_owner->_nomail_image.c_str(), 0);
  gtk_image_set_from_animation (GTK_IMAGE (gtk_bin_get_child (GTK_BIN(get("nomail_image")))), animation);
  if (animation)
    g_object_unref (animation);
  // Here, we attach the full filename to the widget, taking care of previous attached data
  data = (gchar *) g_object_get_data (G_OBJECT(get("nomail_image")), "_file_");
  if (data)
    g_free (data);
  data = g_strdup (_owner->_nomail_image.c_str());
  g_object_set_data (G_OBJECT(get("nomail_image")), "_file_", gpointer (data));

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("hide_nomail")), _owner->_hide_nomail); 
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("display_title")), _owner->_display_title); 
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("display_date")), _owner->_display_date); 
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("max_sender_size")), _owner->_max_sender_size);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("max_subject_size")), _owner->_max_subject_size);


    
  // Geometry
  if (_owner->_popup_x_sign == SIGN_PLUS)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("popup_x_plus")), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("popup_x_minus")), TRUE);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("popup_x")), _owner->_popup_x);
  if (_owner->_popup_y_sign == SIGN_PLUS)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("popup_y_plus")), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("popup_y_minus")), TRUE);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("popup_y")), _owner->_popup_y);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("popup_decorated")), _owner->_popup_decorated); 

  if (_owner->_biff_x_sign == SIGN_PLUS)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("biff_x_plus")), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("biff_x_minus")), TRUE);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("biff_x")), _owner->_biff_x);
  if (_owner->_biff_y_sign == SIGN_PLUS)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("biff_y_plus")), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("biff_y_minus")), TRUE);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("biff_y")), _owner->_biff_y);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(get("biff_decorated")), _owner->_biff_decorated); 

  gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("max_header_display")), _owner->_max_header_display);



  // Font
  string font = "<span font_desc=\""+string(_owner->_popup_title_font) + string("\">") + string(_owner->_popup_title_font) + string ("</span>");
  gtk_label_set_markup (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("popup_title_font")))), font.c_str());
  font = "<span font_desc=\""+string(_owner->_popup_body_font) + string("\">") + string(_owner->_popup_body_font) + string ("</span>");
  gtk_label_set_markup (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("popup_body_font")))), font.c_str());
  font = "<span font_desc=\""+string(_owner->_biff_title_font) + string("\">") + string(_owner->_biff_title_font) + string ("</span>");
  gtk_label_set_markup (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("biff_title_font")))), font.c_str());
  font = "<span font_desc=\""+string(_owner->_biff_body_font) + string("\">") + string(_owner->_biff_body_font) + string ("</span>");
  gtk_label_set_markup (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("biff_body_font")))), font.c_str());
  font = "<span font_desc=\""+string(_owner->_applet_title_font) + string("\">") + string(_owner->_applet_title_font) + string ("</span>");
  gtk_label_set_markup (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("applet_title_font")))), font.c_str());
  font = "<span font_desc=\""+string(_owner->_applet_body_font) + string("\">") + string(_owner->_applet_body_font) + string ("</span>");
  gtk_label_set_markup (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("applet_body_font")))), font.c_str());
  data = (gchar *) g_object_get_data (G_OBJECT(get("popup_title_color")), "_color_");
  if (data)
    g_free (data);
  data = g_strdup (_owner->_popup_title_color.c_str());
  g_object_set_data (G_OBJECT(get("popup_title_color")), "_color_", gpointer (data));
  data = (gchar *) g_object_get_data (G_OBJECT(get("popup_body_color")), "_color_");
  if (data)
    g_free (data);
  data = g_strdup (_owner->_popup_body_color.c_str());
  g_object_set_data (G_OBJECT(get("popup_body_color")), "_color_", gpointer (data));
  data = (gchar *) g_object_get_data (G_OBJECT(get("biff_title_color")), "_color_");
  if (data)
    g_free (data);
  data = g_strdup (_owner->_biff_title_color.c_str());
  g_object_set_data (G_OBJECT(get("biff_title_color")), "_color_", gpointer (data));
  data = (gchar *) g_object_get_data (G_OBJECT(get("biff_body_color")), "_color_");
  if (data)
    g_free (data);
  data = g_strdup (_owner->_biff_body_color.c_str());
  g_object_set_data (G_OBJECT(get("biff_body_color")), "_color_", gpointer (data));
  data = (gchar *) g_object_get_data (G_OBJECT(get("applet_title_color")), "_color_");
  if (data)
    g_free (data);
  data = g_strdup (_owner->_applet_title_color.c_str());
  g_object_set_data (G_OBJECT(get("applet_title_color")), "_color_", gpointer (data));
  data = (gchar *) g_object_get_data (G_OBJECT(get("applet_body_color")), "_color_");
  if (data)
    g_free (data);
  data = g_strdup (_owner->_applet_body_color.c_str());
  g_object_set_data (G_OBJECT(get("applet_body_color")), "_color_", gpointer (data));



  // Now we call some callbacks to have some widgets sensitive state updated
  on_protocol();
  on_sound ();
  on_misc_change ();
}


// ===================================================================
// = updateOwner =====================================================
/**
   update _owner according to widgets content
 */
// ===================================================================
void Setup::updateOwner (void) {
  //
  // Nothing special, we read widget content and set _owner (Biff)
  //  accordingly.
  // Still very boring code...
  //
  // Mailbox
  _owner->_title =gtk_entry_get_text (GTK_ENTRY (get("title")));

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("file")))) {
    if (_owner->_protocol != PROTOCOL_FILE) {
      delete _owner->_mailbox;
      _owner->_mailbox = new File (_owner);
      _owner->_protocol = PROTOCOL_FILE;
    }
  }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("mh")))) {
    if (_owner->_protocol != PROTOCOL_MH) {
      delete _owner->_mailbox;
      _owner->_mailbox = new Mh (_owner);
      _owner->_protocol = PROTOCOL_MH;
    }
  }
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("maildir")))) {
    if (_owner->_protocol != PROTOCOL_MAILDIR) {
      delete _owner->_mailbox;
      _owner->_mailbox = new Maildir (_owner);
      _owner->_protocol = PROTOCOL_MAILDIR;
    }
  }
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("pop3")))) {
    if (_owner->_protocol != PROTOCOL_POP3) {
      delete _owner->_mailbox;
      _owner->_mailbox = new Pop3 (_owner);
      _owner->_protocol = PROTOCOL_POP3;
    }
  }
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("apop")))) {
    if (_owner->_protocol != PROTOCOL_APOP) {
      delete _owner->_mailbox;
      _owner->_mailbox = new Apop (_owner);
      _owner->_protocol = PROTOCOL_APOP;
    }
  }
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("imap4")))) {
    if (_owner->_protocol != PROTOCOL_IMAP4) {
      delete _owner->_mailbox;
      _owner->_mailbox = new Imap4 (_owner);
      _owner->_protocol = PROTOCOL_IMAP4;
    }
  }
  _owner->_port = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(get("port")));
  _owner->_ssl = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("ssl")));
  _owner->_address =gtk_entry_get_text (GTK_ENTRY (get("address")));
  _owner->_user = gtk_entry_get_text (GTK_ENTRY (get("user")));
  _owner->_password =  gtk_entry_get_text (GTK_ENTRY (get("password")));
  _owner->_certificate =  gtk_entry_get_text (GTK_ENTRY (get("certificate")));


  // Options
  _owner->_polltime =  gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(get("polltime")));
  _owner->_poptime =  gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(get("poptime")));
  _owner->_sound_type = SOUND_NONE;
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("beep"))))
    _owner->_sound_type = SOUND_BEEP;
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("sound"))))
    _owner->_sound_type = SOUND_FILE;
  gchar *data = (gchar *) g_object_get_data (G_OBJECT(get("soundfile")), "_file_");
  _owner->_sound_file = data;
  _owner->_sound_volume = (int) (gtk_range_get_value (GTK_RANGE (get ("volume"))));



  // Advanced
  _owner->_display_popup = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("display_popup")));
  _owner->_mail_app =  gtk_entry_get_text (GTK_ENTRY (get("mail_app")));



  // Layout
  data = (gchar *) g_object_get_data (G_OBJECT(get("mail_image")), "_file_");
  _owner->_mail_image = data;
  data = (gchar *) g_object_get_data (G_OBJECT(get("nomail_image")), "_file_");
  _owner->_nomail_image = data;
  _owner->_hide_nomail = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("hide_nomail")));
  _owner->_display_title = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("display_title")));
  _owner->_display_date = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("display_date")));
  _owner->_max_sender_size =  gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(get("max_sender_size")));
  _owner->_max_subject_size =  gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(get("max_subject_size")));



  // Geometry
  _owner->_popup_x_sign = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("popup_x_plus")));
  _owner->_popup_x = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(get("popup_x")));
  _owner->_popup_y_sign = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("popup_y_plus")));
  _owner->_popup_y = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(get("popup_y")));
  _owner->_popup_decorated = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("popup_decorated")));
  _owner->_biff_x_sign = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("biff_x_plus")));
  _owner->_biff_x = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(get("biff_x")));
  _owner->_biff_y_sign = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("biff_y_plus")));
  _owner->_biff_y = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(get("biff_y")));
  _owner->_biff_decorated = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("biff_decorated")));
  _owner->_max_header_display = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(get("max_header_display")));

  

  // Font
  // We font name directly from button label
  _owner->_popup_title_font = gtk_label_get_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("popup_title_font")))));
  _owner->_popup_body_font = gtk_label_get_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("popup_body_font")))));
  _owner->_biff_title_font = gtk_label_get_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("biff_title_font")))));
  _owner->_biff_body_font = gtk_label_get_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("biff_body_font")))));
  _owner->_applet_title_font = gtk_label_get_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("applet_title_font")))));
  _owner->_applet_body_font = gtk_label_get_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("applet_body_font")))));

  data = (gchar *) g_object_get_data (G_OBJECT(get("popup_title_color")), "_color_");
  _owner->_popup_title_color = data;
  data = (gchar *) g_object_get_data (G_OBJECT(get("popup_body_color")), "_color_");
  _owner->_popup_body_color = data;

  data = (gchar *) g_object_get_data (G_OBJECT(get("biff_title_color")), "_color_");
  _owner->_biff_title_color = data;
  data = (gchar *) g_object_get_data (G_OBJECT(get("biff_body_color")), "_color_");
  _owner->_biff_body_color = data;
  
  data = (gchar *) g_object_get_data (G_OBJECT(get("applet_title_color")), "_color_");
  _owner->_applet_title_color = data;
  data = (gchar *) g_object_get_data (G_OBJECT(get("applet_body_color")), "_color_");
  _owner->_applet_body_color = data;
}


// ===================================================================
// = on_delete =======================================================
/**
   Callback on delete event: all parameters are dropped and the
   setting dialog is hidden.
   @param widget where event occured
   @param event is the delete event
*/
// ===================================================================
gboolean Setup::on_delete (GtkWidget *widget,  GdkEvent *event) {
  hide();
  update();
  _owner->applet()->update();
  _owner->applet()->show();
  return true;
}


// ===================================================================
// = on_cancel =======================================================
/**
   Callback on cancel button: all parameters are dropped and the
   setting dialog is hidden
*/
// ===================================================================
void Setup::on_cancel (void) {
  hide();
  update ();
  _owner->applet()->update();
  _owner->applet()->show();
  hide();
}


// ===================================================================
// = on_save =========================================================
/**
   Callback on save button: all parameters are updated from GUI and
   saved
*/
// ===================================================================
void Setup::on_save (void) {
  hide();
  updateOwner();
  _owner->save ();
  _owner->applet()->update();
  _owner->applet()->show();
  if (_owner->_state == STATE_RUNNING)
    _owner->applet()->start();
}


// ===================================================================
// = on_quit =========================================================
/**
   Callback on quit button: exit gnubiff
*/
// ===================================================================
void Setup::on_quit (void) {
  gtk_main_quit();
}


// ===================================================================
// = on_suspend ======================================================
/**
   Callback on suspend button: switch to supend or continue state
   (manual or automatic checking)
*/
// ===================================================================
void Setup::on_suspend (void) {
  if (_owner->_state == STATE_RUNNING) {
    gtk_label_set_markup (GTK_LABEL (get("suspend_label")), "<span color=\"red\"><b>Stopped</b></span>");
    gtk_image_set_from_stock (GTK_IMAGE (get("suspend_image")), GTK_STOCK_STOP, GTK_ICON_SIZE_BUTTON);
    _owner->_state = STATE_STOPPED;
  }
  else {
    gtk_label_set_markup (GTK_LABEL (get("suspend_label")), "<span color=\"darkgreen\"><b>Running</b></span>");
    gtk_image_set_from_stock (GTK_IMAGE (get("suspend_image")), GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON);
    _owner->_state = STATE_RUNNING;
  }
}

// ===================================================================
// = on_protocol =====================================================
/**
   Callback on protocol (any) button: when a protocol button is
   pressed, we update user and password sensitive state
*/
// ===================================================================
void Setup::on_protocol (void) {
  gtk_widget_set_sensitive (get("network"), TRUE);
  gtk_widget_set_sensitive (get("port_all"), TRUE);
  gtk_widget_set_sensitive (get("address_browse"), FALSE);
#ifdef HAVE_LIBSSL
  gtk_widget_set_sensitive (get("ssl"), TRUE);
  gtk_widget_set_sensitive (get("CertificateTable"), TRUE);
#else
  gtk_widget_set_sensitive (get("ssl"), FALSE);
  gtk_widget_set_sensitive (get("CertificateTable"), FALSE);
#endif
  
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("file"))) ||
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("mh"))) ||
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("maildir")))) {
    gtk_widget_set_sensitive (get("network"), FALSE);
    gtk_widget_set_sensitive (get("port_all"), FALSE);
    gtk_widget_set_sensitive (get("address_browse"), TRUE);
    gtk_widget_set_sensitive (get("ssl"), FALSE);
    gtk_widget_set_sensitive (get("CertificateTable"), FALSE);
  }

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("ssl"))))
    gtk_image_set_from_file (GTK_IMAGE(get("ssl_image")), GNUBIFF_DATADIR"/stock-ssl.png");
  else
    gtk_image_set_from_file (GTK_IMAGE(get("ssl_image")), GNUBIFF_DATADIR"/stock-nossl.png");

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("imap4")))) {
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("ssl"))))
      gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("port")), DEFAULT_SSL_IMAP4_PORT);
    else
      gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("port")), DEFAULT_IMAP4_PORT);
  }
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("pop3")))) {
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("ssl"))))
      gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("port")), DEFAULT_SSL_POP3_PORT);
    else
      gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("port")), DEFAULT_POP3_PORT);
  }
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("apop")))) {
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("ssl"))))
      gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("port")), DEFAULT_SSL_POP3_PORT);
    else
      gtk_spin_button_set_value (GTK_SPIN_BUTTON(get("port")), DEFAULT_POP3_PORT);
  }

  if(!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("ssl")))){
    gtk_widget_set_sensitive (get("CertificateTable"), FALSE);
  }
}

// ===================================================================
// = on_sound ========================================================
/**
   Callback on sound (any) button: update mailfile, play and volume
   sensitive state
*/
// ===================================================================
void Setup::on_sound (void) {
  gtk_widget_set_sensitive (get("volume_all"), TRUE);
  gtk_widget_set_sensitive (get("soundfile"), TRUE);
  gtk_widget_set_sensitive (get("soundfilelabel"), TRUE);
  gtk_widget_set_sensitive (get("soundplay"), TRUE);

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("nosound")))) {
    gtk_widget_set_sensitive (get("volume_all"), FALSE);
    gtk_widget_set_sensitive (get("soundfile"), FALSE);
    gtk_widget_set_sensitive (get("soundfilelabel"), FALSE);
    gtk_widget_set_sensitive (get("soundplay"), FALSE);
  }
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("beep")))) {
    gtk_widget_set_sensitive (get("volume_all"), FALSE);
    gtk_widget_set_sensitive (get("soundfile"), FALSE);
    gtk_widget_set_sensitive (get("soundfilelabel"), FALSE);
  }
}


// ===================================================================
// = on_misc_change ==================================================
/**
   Callback on misc. changes
*/
// ===================================================================
void Setup::on_misc_change (void) {
  gtk_widget_set_sensitive (get("title_all"), FALSE);
  gtk_widget_set_sensitive (get("popup_title_font_box"), FALSE);
#ifdef USE_GNOME
  if (_owner->_mode == GNOME_MODE)
    gtk_widget_set_sensitive (get("applet_title_font_box"), FALSE);
  else
    gtk_widget_set_sensitive (get("biff_title_font_box"), FALSE);
#else
  gtk_widget_set_sensitive (get("biff_title_font_box"), FALSE);
#endif

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("display_title")))) {
    gtk_widget_set_sensitive (get("title_all"), TRUE);
    gtk_widget_set_sensitive (get("popup_title_font_box"), TRUE);
#ifdef USE_GNOME
    if (_owner->_mode == GNOME_MODE)
      gtk_widget_set_sensitive (get("applet_title_font_box"), TRUE);
    else
      gtk_widget_set_sensitive (get("biff_title_font_box"), TRUE);
#else
      gtk_widget_set_sensitive (get("biff_title_font_box"), TRUE);
#endif
  }

  gtk_widget_set_sensitive (get("popup_geometry_table"), TRUE);
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("popup_no_geometry"))))
    gtk_widget_set_sensitive (get("popup_geometry_table"), FALSE);
#ifndef USE_GNOME
  gtk_widget_set_sensitive (get("biff_geometry_table"), TRUE);
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("biff_no_geometry"))))
    gtk_widget_set_sensitive (get("biff_geometry_table"), FALSE);
#else
  if (_owner->_mode == GTK_MODE) {
    gtk_widget_set_sensitive (get("biff_geometry_table"), TRUE);
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("biff_no_geometry"))))
      gtk_widget_set_sensitive (get("biff_geometry_table"), FALSE);
  }
#endif
}


// ===================================================================
// = on_browse_address ===============================================
/**
   Callback on address browse button: open a file selector to browse
   for a file
*/
// ===================================================================
void Setup::on_browse_address (void) {
  GtkWidget *selector = gtk_file_selection_new ("Browse for mailbox file/directory");
  gtk_window_set_position (GTK_WINDOW (selector), GTK_WIN_POS_CENTER);
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (selector), gtk_entry_get_text (GTK_ENTRY (get("address"))));
  gint result = gtk_dialog_run (GTK_DIALOG(selector));
  if (result == GTK_RESPONSE_OK)
    gtk_entry_set_text (GTK_ENTRY (get("address")), gtk_file_selection_get_filename (GTK_FILE_SELECTION (selector)));
  gtk_widget_destroy (selector);
  return;
}

// ===================================================================
// = on_browse_certificate============================================
/**
   Callback on certificate browse button: open a file selector to 
   browse for a file
*/
// ===================================================================
void Setup::on_browse_certificate (void) {
  GtkWidget *selector = gtk_file_selection_new ("Browse for server certificate");
  gtk_window_set_position (GTK_WINDOW (selector), GTK_WIN_POS_CENTER);
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (selector), gtk_entry_get_text (GTK_ENTRY (get("certificate"))));
  gint result = gtk_dialog_run (GTK_DIALOG(selector));
  if (result == GTK_RESPONSE_OK)
    gtk_entry_set_text (GTK_ENTRY (get("certificate")), gtk_file_selection_get_filename (GTK_FILE_SELECTION (selector)));
  gtk_widget_destroy (selector);
  return;
}

// ===================================================================
// = on_browse_sound =================================================
/**
   Callback on sound browse button: open a file selector to browse for
   a soundfile
*/
// ===================================================================
void Setup::on_browse_sound (void) {
  GtkWidget *selector = gtk_file_selection_new ("Open a soundfile");
  gtk_window_set_position (GTK_WINDOW (selector), GTK_WIN_POS_CENTER);
  gchar *data = (gchar *) g_object_get_data (G_OBJECT(get("soundfile")), "_file_");
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (selector), data);
  gint result = gtk_dialog_run (GTK_DIALOG(selector));
  if (result == GTK_RESPONSE_OK) {
    g_free (data);
    data = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (selector)));
    g_object_set_data (G_OBJECT(get("soundfile")), "_file_", gpointer (data));
    gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get("soundfile")))),
			g_path_get_basename (data));
  }
  gtk_widget_destroy (selector);
}


// ===================================================================
// = on_play_sound ===================================================
/**
   Callback on sound play button: play current soundfile or beep
*/
// ===================================================================
void Setup::on_play_sound (void) {
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("beep"))))
    gdk_beep ();    
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("sound")))) {
    gchar *data = (gchar *) g_object_get_data (G_OBJECT(get("soundfile")), "_file_");
    stringstream s;
    s << gfloat(gtk_range_get_value (GTK_RANGE (get ("volume"))))/100.0;
    string command = "play " + string (data) + string (" -v ");
    command += s.str();
    system (command.c_str());
  }
}


// ===================================================================
// = on_browse_mail_app ==============================================
/**
   Callback on mail_app browse button: open a file selector to browse
   for a mail app
*/
// ===================================================================
void Setup::on_browse_mail_app (void) {
  GtkWidget *selector = gtk_file_selection_new ("Open a mail application");
  gtk_window_set_position (GTK_WINDOW (selector), GTK_WIN_POS_CENTER);
  gchar *data = (gchar *) g_object_get_data (G_OBJECT(get("mail_app")), "_file_");
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (selector), data);
  gint result = gtk_dialog_run (GTK_DIALOG(selector));
  if (result == GTK_RESPONSE_OK) {
    g_free (data);
    data = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (selector)));
    g_object_set_data (G_OBJECT(get("mail_app")), "_file_", gpointer (data));
    gtk_entry_set_text (GTK_ENTRY (get("mail_app")), g_path_get_basename (data));
  }
  gtk_widget_destroy (selector);
}

// ===================================================================
// = on_browse_mail_image ============================================
/**
   Callback on mail_image browse button: open a file selector to
   browse for a mail image
*/
// ===================================================================
void Setup::on_browse_mail_image (void) {
  GtkWidget *selector = gtk_file_selection_new ("Open a mail image or animation");
  gtk_window_set_position (GTK_WINDOW (selector), GTK_WIN_POS_CENTER);
  gchar *data = (gchar *) g_object_get_data (G_OBJECT(get("mail_image")), "_file_");
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (selector), data);
  gint result = gtk_dialog_run (GTK_DIALOG(selector));
  if (result == GTK_RESPONSE_OK) {
    g_free (data);
    data = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (selector)));
    g_object_set_data (G_OBJECT(get("mail_image")), "_file_", gpointer (data));

    GdkPixbufAnimation *animation = gdk_pixbuf_animation_new_from_file (data, 0);
    gtk_image_set_from_animation (GTK_IMAGE (gtk_bin_get_child (GTK_BIN(get("mail_image")))), animation);
    if (animation)
      g_object_unref (animation);
  }
  gtk_widget_destroy (selector);
}

// ===================================================================
// = on_browse_nomail_image ==========================================
/**
   Callback on nomail_image browse button: open a file selector to
   browse for a nomail image
*/
// ===================================================================
void Setup::on_browse_nomail_image (void) {
  GtkWidget *selector = gtk_file_selection_new ("Open a nomail image or animation");
  gtk_window_set_position (GTK_WINDOW (selector), GTK_WIN_POS_CENTER);
  gchar *data = (gchar *) g_object_get_data (G_OBJECT(get("nomail_image")), "_file_");
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (selector), data);
  gint result = gtk_dialog_run (GTK_DIALOG(selector));
  if (result == GTK_RESPONSE_OK) {
    g_free (data);
    data = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (selector)));
    g_object_set_data (G_OBJECT(get("nomail_image")), "_file_", gpointer (data));

    GdkPixbufAnimation *animation = gdk_pixbuf_animation_new_from_file (data, 0);
    gtk_image_set_from_animation (GTK_IMAGE (gtk_bin_get_child (GTK_BIN(get("nomail_image")))), animation);
    if (animation)
      g_object_unref (animation);
  }
  gtk_widget_destroy (selector);
}


// ===================================================================
// = on_browse_color =================================================
/**
   Generic color browser
*/
// ===================================================================
void Setup::on_browse_color (string name) {
  GtkColorSelectionDialog *selector = GTK_COLOR_SELECTION_DIALOG (gtk_color_selection_dialog_new ("Pick a color"));
  gtk_window_set_position (GTK_WINDOW (selector), GTK_WIN_POS_CENTER);
  gchar *data = (gchar *) g_object_get_data (G_OBJECT(get(name.c_str())), "_color_");
  GdkColor color;
  gdk_color_parse (data, &color);
  gtk_color_selection_set_current_color (GTK_COLOR_SELECTION (selector->colorsel), &color);

  gint result = gtk_dialog_run (GTK_DIALOG(selector));
  if (result == GTK_RESPONSE_OK) {
    g_free (data);
    gtk_color_selection_get_current_color (GTK_COLOR_SELECTION (selector->colorsel), &color);
    char buffer[16];
    sprintf (buffer, "#%.2x%.2x%.2x", color.red/256, color.green/256, color.blue/256);
    data = g_strdup (buffer);
    g_object_set_data (G_OBJECT(get(name.c_str())), "_color_", gpointer (data));
  }
  gtk_widget_destroy (GTK_WIDGET(selector));
}


// ===================================================================
// = on_browse_font ==================================================
/**
   Generic font browser
*/
// ===================================================================
void Setup::on_browse_font (string name) {
  GtkFontSelectionDialog *selector = GTK_FONT_SELECTION_DIALOG (gtk_font_selection_dialog_new ("Pick a font"));
  gtk_window_set_position (GTK_WINDOW (selector), GTK_WIN_POS_CENTER);
  gtk_font_selection_dialog_set_font_name (selector, gtk_label_get_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get(name.c_str()))))));
  gint result = gtk_dialog_run (GTK_DIALOG(selector));
  if (result == GTK_RESPONSE_OK) {
    string font = "<span font_desc=\""+string(gtk_font_selection_dialog_get_font_name (selector)) + string("\">")
      + string(gtk_font_selection_dialog_get_font_name (selector)) + string ("</span>");
    gtk_label_set_markup (GTK_LABEL (gtk_bin_get_child (GTK_BIN(get(name.c_str())))), font.c_str());
  }
  gtk_widget_destroy (GTK_WIDGET(selector));
}


// ===================================================================
// = on_tree_select ==================================================
/**
   Callback on tree selection: display the corresponding notebook page
   and update title and icon above the page
   @param sel is the selection
 */
// ===================================================================
void Setup::on_tree_select (GtkTreeSelection *sel)  {
  GtkTreeModel *model;
  GtkTreeIter   iter;
  GValue        val = {0,};

  if (!gtk_tree_selection_get_selected (sel, &model, &iter))
    return;
  gtk_tree_model_get_value (model, &iter, 3, &val);

  string label = "<span size=\"x-large\"><b>";
  label += g_value_get_string (&val);
  label += "</b></span>";
  gtk_label_set_markup (GTK_LABEL (get("NotebookLabel")),  label.c_str());

  g_value_unset (&val);
  gtk_tree_model_get_value (model, &iter, 4, &val);
  gtk_image_set_from_pixbuf (GTK_IMAGE (get("NotebookImage")), (GdkPixbuf *) g_value_get_object (&val));
  g_value_unset (&val);
  gtk_tree_model_get_value (model, &iter, 2, &val);
  gtk_notebook_set_current_page (GTK_NOTEBOOK (get("Notebook")), g_value_get_int (&val));
  g_value_unset (&val);
}


// ===================================================================
// = addTreeBranch ===================================================
/**
   Add a new setting to the tree 
   @param tree is the tree store
   @param notebook_label is the notebook page label
   @param icon is the icon for both tree and notebook
   @param tree_label is the new tree label
   @param parent is the parent node
   @param iter is the current iteration (for possible children)
   @param page_index is the index of corresponding notebook page
*/
// ===================================================================
void
Setup::addTreeBranch (GtkTreeStore *tree,
		      gchar *tree_label,
		      const gchar *icon,
		      gchar *notebook_label,
		      GtkTreeIter *parent,
		      GtkTreeIter *iter,
		      gint page_index) {
  GdkPixbuf   *pixbuf       = NULL;
  GdkPixbuf   *big_pixbuf   = NULL;
  GdkPixbuf   *small_pixbuf = NULL;

  if (icon)  {
    if (g_file_test (icon, G_FILE_TEST_IS_REGULAR))
      pixbuf = gdk_pixbuf_new_from_file (icon, NULL);
    else
      pixbuf = NULL;
    if (pixbuf) {
      big_pixbuf = gdk_pixbuf_scale_simple (pixbuf, 48, 48, GDK_INTERP_BILINEAR);
      small_pixbuf = gdk_pixbuf_scale_simple (pixbuf, 18, 18, GDK_INTERP_BILINEAR);
    }
  }
  gtk_tree_store_append (tree, iter, parent);
  gtk_tree_store_set (tree, iter,
                      0, small_pixbuf,
                      1, tree_label,
                      2, page_index,
                      3, notebook_label,
                      4, big_pixbuf,
                      -1);
  if (pixbuf)
    g_object_unref (pixbuf);
  if (small_pixbuf)
    g_object_unref (small_pixbuf);
  if (big_pixbuf)
    g_object_unref (big_pixbuf);
}
